@@ -1,4 +1,4 @@
-;;; spatial-navigate.el --- Directional navigation between white-space blocks -*- lexical-binding: t -*-
+;;; spatial-navigate.el --- Directional navigation between blank-space blocks -*- lexical-binding: t -*-
 
 ;; SPDX-License-Identifier: GPL-2.0-or-later
 ;; Copyright (C) 2020  Campbell Barton
@@ -6,27 +6,26 @@
 ;; Author: Campbell Barton <ideasman42@gmail.com>
 
 ;; URL: https://codeberg.org/ideasman42/emacs-spatial-navigate
-;; Package-Version: 20251126.513
-;; Package-Revision: 51aee2673323
+;; Package-Version: 20251229.1145
+;; Package-Revision: 9f61d3894c2f
 ;; Package-Requires: ((emacs "29.1"))
 
 ;;; Commentary:
 
-;; Support jumping horizontally & vertically
-;; across blocks of white-space or non-white-space.
+;; Support jumping horizontally and vertically
+;; across blocks of blank-space or non-blank-space.
 
-;;; Usage
+;;; Usage:
 
-
-;; ;; This shows how using Alt-Arrow Keys can be used to set directional navigation.
+;; ;; This shows how Alt-arrow keys can be used for directional navigation.
 ;; (global-set-key (kbd "<M-up>") 'spatial-navigate-backward-vertical-bar)
 ;; (global-set-key (kbd "<M-down>") 'spatial-navigate-forward-vertical-bar)
 ;; (global-set-key (kbd "<M-left>") 'spatial-navigate-backward-horizontal-bar)
 ;; (global-set-key (kbd "<M-right>") 'spatial-navigate-forward-horizontal-bar)
 
 ;; ;; If you use evil-mode, the 'box' navigation functions make sense in normal mode,
-;; ;; the 'bar' functions make most sense in insert mode.
-;;
+;; ;; while the 'bar' functions make the most sense in insert mode.
+
 ;; (define-key evil-normal-state-map (kbd "M-k") 'spatial-navigate-backward-vertical-box)
 ;; (define-key evil-normal-state-map (kbd "M-j") 'spatial-navigate-forward-vertical-box)
 ;; (define-key evil-normal-state-map (kbd "M-h") 'spatial-navigate-backward-horizontal-box)
@@ -38,7 +37,6 @@
 
 ;;; Code:
 
-
 ;; ---------------------------------------------------------------------------
 ;; Compatibility
 
@@ -66,10 +64,10 @@
 
 
 ;; ---------------------------------------------------------------------------
-;; Generic Functions
+;; Evil Mode Support
 
 (defun spatial-navigate--evil-visual-mode-workaround (state)
-  "Workaround for evil-visual line mode, STATE must be \\'pre or \\'post."
+  "Workaround for evil-visual line mode, STATE must be \\='pre or \\='post."
   (declare (important-return-value nil))
   (when (and (fboundp 'evil-visual-state-p)
              (funcall #'evil-visual-state-p)
@@ -82,8 +80,8 @@
          ;; Without this, `point' will be at the beginning of the line
          ;; (from the pre command hook).
          ((eq state 'pre)
-          (goto-char (marker-position mark)))
-         ;; Without this, the `point' wont move.
+          (goto-char mark))
+         ;; Without this, the `point' won't move.
          ;; See: https://github.com/emacs-evil/evil/issues/1708
          ((eq state 'post)
           (set-marker mark (point)))
@@ -94,12 +92,21 @@
 ;; ---------------------------------------------------------------------------
 ;; Private Functions
 
+(defun spatial-navigate--char-filled-p (pos beg end default)
+  "Return non-nil if POS contains a non-blank-space character.
+BEG and END are line bounds.  DEFAULT is returned if POS is out of range."
+  (declare (important-return-value t))
+  (if (and (>= pos beg) (< pos end))
+      (let ((ch (char-after pos)))
+        (not (memq ch '(?\s ?\t))))
+    default))
+
 (defun spatial-navigate--vertical-calc (dir is-block-cursor)
   "Calculate the next/previous vertical position based on DIR (-1 or 1).
 
-Argument IS-BLOCK-CURSOR causes the cursor to detect white-space using
-characters before and after the current cursor, this behaves in a way that
-is logical for a block cursor)."
+Argument IS-BLOCK-CURSOR causes the cursor to detect blank-space using
+characters before and after the current cursor.  This behaves in a way that
+is logical for a block cursor."
   (declare (important-return-value t))
   (spatial-navigate--evil-visual-mode-workaround 'pre)
 
@@ -110,18 +117,10 @@ is logical for a block cursor)."
         (is-first t)
         (is-empty-state nil)
         (pos-prev (point))
-        (col-init (current-column))
-        (is-fill-fn
-         (lambda (pos beg end default)
-           (cond
-            ((and (>= pos beg) (< pos end))
-             (let ((ch (char-after pos)))
-               (null (memq ch (list ?\s ?\t)))))
-            (t
-             default)))))
+        (col-init (current-column)))
     (while (null result)
 
-      ;; Forward line and move to column.
+      ;; Step to next line and move to column.
       (forward-line dir)
       (incf lines dir)
 
@@ -131,24 +130,27 @@ is logical for a block cursor)."
                   ;; End of the line is also considered empty.
                   (and (or (zerop col-init) is-block-cursor) (eolp))
 
-                  ;; Do this so we don't delimit on spaces between words.
-                  ;; Surrounded by spaces before and after.
+                  ;; Avoid delimiting on spaces between words by checking
+                  ;; if we're surrounded by spaces before and after.
                   (let* ((pos-eol (pos-eol))
                          (pos-bol (pos-bol))
 
-                         (is-fill-curr (funcall is-fill-fn (point) pos-bol pos-eol nil))
+                         (is-fill-curr
+                          (spatial-navigate--char-filled-p (point) pos-bol pos-eol nil))
                          (is-fill-prev
-                          (funcall is-fill-fn (+ (point) 1) pos-bol pos-eol is-fill-curr))
+                          (spatial-navigate--char-filled-p
+                           (- (point) 1) pos-bol pos-eol is-fill-curr))
                          (is-fill-next
-                          (funcall is-fill-fn (- (point) 1) pos-bol pos-eol is-fill-curr)))
+                          (spatial-navigate--char-filled-p
+                           (+ (point) 1) pos-bol pos-eol is-fill-curr)))
 
                     (cond
-                     ;; Check 3 characters, current char, before & after.
-                     ;; If there are two blanks before or after, this is considered not filled.
+                     ;; Check three characters: current char, before, and after.
+                     ;; Empty if current is blank and at least one neighbor is also blank.
                      (is-block-cursor
                       (null (or is-fill-curr (and is-fill-prev is-fill-next))))
 
-                     ;; Check only 2 characters.
+                     ;; Check only two characters: current and previous.
                      (t
                       (null (or is-fill-curr is-fill-prev))))))))
 
@@ -168,8 +170,7 @@ is logical for a block cursor)."
                  (t
                   (cons lines-prev pos-prev)))))
          ((eq pos-prev (point))
-          ;; Beginning or end, don't hang!
-          ;; Use the last valid state.
+          ;; Point didn't move, we're at buffer boundary.
           (setq result result-fallback))
          (t ; Keep looping.
           ;; If we reach the beginning or end of the document,
@@ -182,11 +183,11 @@ is logical for a block cursor)."
 
 
 (defun spatial-navigate--horizontal-calc (dir is-block-cursor)
-  "Calculate the next/previous vertical position based on DIR (-1 or 1).
+  "Calculate the next/previous horizontal position based on DIR (-1 or 1).
 
-Argument IS-BLOCK-CURSOR causes the cursor to detect white-space using
-characters before and after the current cursor, this behaves in a way that
-is logical for a block cursor)."
+Argument IS-BLOCK-CURSOR causes the cursor to detect blank-space using
+characters before and after the current cursor.  This behaves in a way that
+is logical for a block cursor."
   (declare (important-return-value t))
   (spatial-navigate--evil-visual-mode-workaround 'pre)
 
@@ -196,18 +197,10 @@ is logical for a block cursor)."
         (pos-prev (point))
 
         (pos-eol (pos-eol))
-        (pos-bol (pos-bol))
+        (pos-bol (pos-bol)))
 
-        (is-fill-fn
-         (lambda (pos beg end default)
-           (cond
-            ((and (>= pos beg) (< pos end))
-             (let ((ch (char-after pos)))
-               (null (memq ch (list ?\s ?\t)))))
-            (t
-             default)))))
-
-    ;; This is needed once at the start, unlike line stepping.
+    ;; Initial step is needed here because forward-char requires explicit call,
+    ;; whereas forward-line in vertical-calc implicitly moves to the next line.
     (when (cond
            ((< dir 0)
             (> pos-prev pos-bol))
@@ -217,11 +210,13 @@ is logical for a block cursor)."
 
     (while (null result)
       (let ((is-empty
-             ;; Do this so we don't delimit on spaces between words.
-             ;; Surrounded by spaces before and after.
-             (let* ((is-fill-curr (funcall is-fill-fn (point) pos-bol pos-eol nil))
-                    (is-fill-prev (funcall is-fill-fn (+ (point) 1) pos-bol pos-eol is-fill-curr))
-                    (is-fill-next (funcall is-fill-fn (- (point) 1) pos-bol pos-eol is-fill-curr)))
+             ;; Avoid delimiting on spaces between words by checking
+             ;; if we're surrounded by spaces before and after.
+             (let* ((is-fill-curr (spatial-navigate--char-filled-p (point) pos-bol pos-eol nil))
+                    (is-fill-prev
+                     (spatial-navigate--char-filled-p (- (point) 1) pos-bol pos-eol is-fill-curr))
+                    (is-fill-next
+                     (spatial-navigate--char-filled-p (+ (point) 1) pos-bol pos-eol is-fill-curr)))
                (null (or is-fill-curr (and is-fill-prev is-fill-next))))))
 
         ;; Keep searching for whatever we encounter first.
@@ -248,8 +243,7 @@ is logical for a block cursor)."
                    (t
                     pos-prev))))))
          ((eq pos-prev (point))
-          ;; Beginning or end, don't hang!
-          ;; Use the last valid state.
+          ;; Point didn't move, we're at buffer boundary.
           (setq result (point)))
          ;; If we get out of range, use last usable point.
          ((cond
@@ -257,18 +251,17 @@ is logical for a block cursor)."
             (< (point) pos-bol))
            (t
             (>= (point) pos-eol)))
-          ;; Beginning or end, don't hang!
-          ;; Use the last valid state.
+          ;; Point moved past line bounds, use previous valid position.
           (setq result pos-prev))
          (t ; Keep looping.
-          ;; If we reach the beginning or end of the document, we may need to use this.
+          ;; If we reach the beginning or end of the line, we may need to use this.
           (setq pos-prev (point))
           (when (cond
                  ((< dir 0)
                   (> pos-prev pos-bol))
                  (t
                   (<= pos-prev pos-eol)))
-            ;; Forward character.
+            ;; Step to next character.
             (forward-char dir))))))
     result))
 
@@ -279,9 +272,9 @@ is logical for a block cursor)."
 (defun spatial-navigate--vertical (dir is-block-cursor)
   "See `spatial-navigate--vertical-calc' for docs on DIR and IS-BLOCK-CURSOR.
 
-When DIR is outside -1/1 range, motion will run multiple times.
+When DIR is outside the -1/1 range, motion will run multiple times.
 
-Return the number of steps remaining (0 when all succeed
+Return the number of steps remaining (0 when all succeed)
 or nil when the motion could not be performed."
   (declare (important-return-value nil))
   (let ((times (abs dir))
@@ -360,7 +353,7 @@ or nil when the motion could not be performed."
       (spatial-navigate--evil-visual-mode-workaround 'post)
       (* times dir))
      (t
-      (message "Spatial-navigate: line limit reached!")
+      (message "Spatial-navigate: boundary reached!")
       nil))))
 
 
@@ -371,7 +364,8 @@ or nil when the motion could not be performed."
 
 ;;;###autoload
 (defun spatial-navigate-forward-vertical-box (arg)
-  "Jump forward vertically ARG times across white-space & non-white-space.
+  "Jump forward vertically ARG times across blank-space and non-blank-space.
+A negative ARG reverses the motion.
 Return the number of steps remaining (0 when all succeed)
 or nil when the motion could not be performed.
 
@@ -388,7 +382,8 @@ Use for a box cursor."
 
 ;;;###autoload
 (defun spatial-navigate-backward-vertical-box (arg)
-  "Jump backward vertically ARG times across white-space and non-white-space.
+  "Jump backward vertically ARG times across blank-space and non-blank-space.
+A negative ARG reverses the motion.
 Return the number of steps remaining (0 when all succeed)
 or nil when the motion could not be performed.
 
@@ -405,7 +400,8 @@ Use for a box cursor."
 
 ;;;###autoload
 (defun spatial-navigate-forward-vertical-bar (arg)
-  "Jump forward vertically ARG times across white-space and non-white-space.
+  "Jump forward vertically ARG times across blank-space and non-blank-space.
+A negative ARG reverses the motion.
 Return the number of steps remaining (0 when all succeed)
 or nil when the motion could not be performed.
 
@@ -422,7 +418,8 @@ Use for a bar cursor."
 
 ;;;###autoload
 (defun spatial-navigate-backward-vertical-bar (arg)
-  "Jump backward vertically ARG times across white-space and non-white-space.
+  "Jump backward vertically ARG times across blank-space and non-blank-space.
+A negative ARG reverses the motion.
 Return the number of steps remaining (0 when all succeed)
 or nil when the motion could not be performed.
 
@@ -441,8 +438,8 @@ Use for a bar cursor."
 
 ;;;###autoload
 (defun spatial-navigate-forward-horizontal-box (arg)
-  "Jump forward horizontal across white-space and non-white-space.
-A negative ARG reversed the motion.
+  "Jump forward horizontally ARG times across blank-space and non-blank-space.
+A negative ARG reverses the motion.
 Return the number of steps remaining (0 when all succeed)
 or nil when the motion could not be performed.
 
@@ -459,8 +456,8 @@ Use for a box cursor."
 
 ;;;###autoload
 (defun spatial-navigate-backward-horizontal-box (arg)
-  "Jump backward horizontal across white-space and non-white-space.
-A negative ARG reversed the motion.
+  "Jump backward horizontally ARG times across blank-space and non-blank-space.
+A negative ARG reverses the motion.
 Return the number of steps remaining (0 when all succeed)
 or nil when the motion could not be performed.
 
@@ -477,8 +474,8 @@ Use for a box cursor."
 
 ;;;###autoload
 (defun spatial-navigate-forward-horizontal-bar (arg)
-  "Jump forward horizontal across white-space and non-white-space.
-A negative ARG reversed the motion.
+  "Jump forward horizontally ARG times across blank-space and non-blank-space.
+A negative ARG reverses the motion.
 Return the number of steps remaining (0 when all succeed)
 or nil when the motion could not be performed.
 
@@ -495,8 +492,8 @@ Use for a bar cursor."
 
 ;;;###autoload
 (defun spatial-navigate-backward-horizontal-bar (arg)
-  "Jump backward horizontal across white-space and non-white-space.
-A negative ARG reversed the motion.
+  "Jump backward horizontally ARG times across blank-space and non-blank-space.
+A negative ARG reverses the motion.
 Return the number of steps remaining (0 when all succeed)
 or nil when the motion could not be performed.
 
